/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.app.Activity
import android.content.Context
import android.graphics.Bitmap
import android.net.Uri
import com.google.gson.Gson
import com.google.gson.JsonObject
import io.flutter.plugin.common.MethodCall
import io.scanbot.genericdocument.entity.*
import io.scanbot.genericdocument.entity.GenericDocumentLibrary.wrap
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.flutter.utils.ObjectMapper
import io.scanbot.sdk.flutter.utils.StorageUtils
import io.scanbot.sdk.ui.result.ResultRepository
import io.scanbot.sdk.ui.result.ResultWrapper
import io.scanbot.sdk.ui.view.barcode.batch.configuration.BatchBarcodeScannerConfiguration
import io.scanbot.sdk.ui.view.barcode.configuration.BarcodeScannerConfiguration
import io.scanbot.sdk.ui.view.base.configuration.OrientationMode
import io.scanbot.sdk.ui.view.camera.configuration.DocumentScannerConfiguration
import io.scanbot.sdk.ui.view.edit.configuration.CroppingConfiguration
import io.scanbot.sdk.ui.view.genericdocument.configuration.GenericDocumentRecognizerConfiguration
import io.scanbot.sdk.ui.view.hic.configuration.HealthInsuranceCardScannerConfiguration
import io.scanbot.sdk.ui.view.licenseplate.configuration.LicensePlateScannerConfiguration
import io.scanbot.sdk.ui.view.mrz.configuration.MRZScannerConfiguration
import org.json.JSONArray
import org.json.JSONObject

fun readEhicScannerConfiguration(
    call: MethodCall,
): HealthInsuranceCardScannerConfiguration {
    val arguments = call.arguments as Json
    return HealthInsuranceCardScannerConfiguration.fromJson(JSONObject(arguments as Map<*, *>).toString())
}

fun readLicensePlateScannerConfiguration(
    call: MethodCall,
): LicensePlateScannerConfiguration {
    val arguments = call.arguments as Json
    return LicensePlateScannerConfiguration.fromJson(JSONObject(arguments as Map<*, *>).toString())
}

fun readBarcodeScannerConfiguration(
    call: MethodCall,
): BarcodeScannerConfiguration {
    val arguments = call.arguments as Json
    return BarcodeScannerConfiguration.fromJson(JSONObject(arguments as Map<*, *>).toString())
}


fun readBatchBarcodeScannerConfiguration(
    call: MethodCall,
): BatchBarcodeScannerConfiguration {
    val arguments = call.arguments as Json
    var configuration: BatchBarcodeScannerConfiguration =
        BatchBarcodeScannerConfiguration.fromJson(JSONObject(arguments as Map<*, *>).toString())
    return configuration
}

fun readCroppingConfiguration(
    call: MethodCall,
    gson: Gson
): CroppingConfiguration {

    val arguments = call.arguments as Json
    val croppingConfiguration = call.argument<Json>("page")?.let {
        val page = it.toPage(gson = gson)
        CroppingConfiguration(page).apply {
            arguments.remove("page")
        }
    } ?: throw IllegalArgumentException("Page object is not set")

    val value = checkOrientationParam(call, arguments)
    value?.let {
        croppingConfiguration.setOrientationLockMode(value)
    }
    ObjectMapper.map(arguments, croppingConfiguration)
    return croppingConfiguration
}

fun readMRZScannerConfiguration(
    call: MethodCall,
): MRZScannerConfiguration {
    val arguments = call.arguments as Json
    return MRZScannerConfiguration.fromJson(JSONObject(arguments as Map<*, *>).toString())
}

private fun checkOrientationParam(call: MethodCall, arguments: Json): OrientationMode? {
    if (call.hasArgument("orientationLockMode")) {
        var cameraOrientationMode: OrientationMode? =
            if (arguments["orientationLockMode"] == "LANDSCAPE") {
                OrientationMode.LANDSCAPE
            } else if (arguments["orientationLockMode"] == "PORTRAIT") {
                OrientationMode.PORTRAIT
            } else {
                null
            }
        arguments.remove("orientationLockMode")
        return cameraOrientationMode
    }
    return null
}

fun readDocumentScannerConfiguration(
    call: MethodCall,
    activity: Activity
): DocumentScannerConfiguration {
    val arguments = call.arguments as Json
    return DocumentScannerConfiguration.fromJson(JSONObject(arguments as Map<*, *>).toString())
}


fun readGenericDocumentRecognizerConfiguration(
    call: MethodCall,
    activity: Activity
): GenericDocumentRecognizerConfiguration {
    val arguments = call.arguments as Json
    return GenericDocumentRecognizerConfiguration.fromJson(JSONObject(arguments as Map<*, *>).toString())
}

fun extractGenericDocumentRecognizerResultFromResultWrappers(
    resultWrappers: java.util.ArrayList<ResultWrapper<GenericDocument>>?,
    scanbotSDK: ScanbotSDK,
    gson: Gson
): JsonObject {
    val outArgs = JsonObject()
    if (resultWrappers == null) {
        return outArgs
    }

    val documents: List<GenericDocument> =
        extractDocumentsFromResultWrappers(resultWrappers, scanbotSDK)

    return gson.toJsonTree( GenericDocumentResult(documents)).asJsonObject
}

fun extractDocumentsFromResultWrappers(
    wrappers: List<ResultWrapper<GenericDocument>>,
    scanbotSDK: ScanbotSDK
): List<GenericDocument> {
    val documents: MutableList<GenericDocument> = java.util.ArrayList()

    for ((resultId, clazz) in wrappers) {
        // Get the ResultRepository from the ScanbotSDK instance
        val resultRepository: ResultRepository<GenericDocument> =
            scanbotSDK.resultRepositoryForClass(clazz)

        // Receive an instance of GenericDocument class from the repository
        // This call will also remove the result from the repository (to make the memory usage less)
        val document = resultRepository.getResultAndErase(resultId) ?: continue
        documents.add(document)
    }
    return documents
}

class GenericDocumentResult(val documents:List<GenericDocument>)
