//
//  Dictionary.swift
//  Pods-Runner
//
//  Created by Zhenya Knizhnik on 23.08.19.
//

import Foundation
import ScanbotSDK

extension Dictionary {
    var withoutNils: Dictionary {
        return self.filter { key, _ in
            self.keys.contains { hash in
                hash == key
            }
        }
    }
    
    var encodedString: String? {
        if let json = try? JSONSerialization.data(withJSONObject: self) {
            let convertedString = String(data: json, encoding: String.Encoding.utf8)
            return convertedString
        }
        return nil
    }
    
    var finderConfiguration: SBSDKBaseScannerViewFinderConfiguration {
        if let finderConfiguration = self["finder" as! Key] as? [String: Any] {
            let finderConfig = SBSDKBaseScannerViewFinderConfiguration.default()
            finderConfig.backgroundColor = UIColor(hexString: "#00000000")
            finderConfig.lineColor = UIColor(hexString: "#00000000")
            finderConfig.isViewFinderEnabled = true
            
            if
                let finderAspectRatio = finderConfiguration["finderAspectRatio"] as? [String: Any],
                let width = finderAspectRatio["width"] as? Double,
                let height = finderAspectRatio["height"] as? Double
            {
                finderConfig.aspectRatio = SBSDKAspectRatio(width: width, andHeight: height)
            }
            
            if let insets = finderConfiguration["finderInsets"] as? [String: Any] {
                if let left = insets["left"] as? CGFloat {
                    finderConfig.minimumInset.left = left
                }
                
                if let top = insets["top"] as? CGFloat {
                    finderConfig.minimumInset.top = top
                }
                
                if let right = insets["right"] as? CGFloat {
                    finderConfig.minimumInset.right = right
                }
                
                if let bottom = insets["bottom"] as? CGFloat {
                    finderConfig.minimumInset.bottom = bottom
                }
            }
           return finderConfig
        }
        return SBSDKBaseScannerViewFinderConfiguration.default()
    }
}
