//
//  SBSDKGenericDocument.swift
//  scanbot_sdk
//
//  Created by Serhii Chaban on 12.07.23.
//

import Foundation
import ScanbotSDK

extension SBSDKGenericDocument {
    var encodedDictionary: [String: Any] {
        
        var dictionary: [String: Any] = [:]
        dictionary["type"] = self.type.encodedDictionary
        
        dictionary["confidence"] = self.confidence
        dictionary["confidenceWeight"] = self.confidenceWeight
        
        var fields = [[String: Any]]()
        for field in self.fields {
            fields.append(field.encodedDictionary)
        }
        dictionary["fields"] = fields
        
        var childs = [[String: Any]]()
        for child in self.children {
            childs.append(child.encodedDictionary)
        }
        dictionary["children"] = childs
        
        return dictionary
    }
}

extension SBSDKGenericDocumentType{
    var encodedDictionary: [String: Any?] {
        var dictionary: [String: Any?] = [:]
        dictionary["name"] = self.name
        dictionary["fullName"] = self.fullName
        dictionary["normalizedName"] = self.normalizedName
        return dictionary
    }
}

extension SBSDKGenericDocumentField{
    var encodedDictionary: [String: Any] {
        var dictionary: [String: Any] = [:]
        dictionary["type"] = self.type.encodedDictionary
        
        if let value = self.value {
            dictionary["value"] = value.encodedDictionary
        }
        
        if(self.validationStatus != SBSDKGenericDocumentFieldValidationStatus.undefined){
            dictionary["validationStatus"] = self.validationStatus.encodedString
        }
        
        return dictionary
    }
}

extension SBSDKGenericDocumentFieldType{
    var encodedDictionary: [String: Any] {
        var dictionary: [String: Any] = [:]
        
        dictionary["name"] = self.name
        dictionary["fullName"] = self.fullName
        dictionary["normalizedName"] = self.normalizedName
        dictionary["commonType"] = self.commonType.encodedString
        
        return dictionary
    }
}

extension SBSDKGenericDocumentOcrResult{
    var encodedDictionary: [String: Any?] {
        var dictionary: [String: Any?] = [:]
        
        dictionary["text"] = self.text
        dictionary["confidence"] = self.confidence
        
        return dictionary
    }
}

extension SBSDKGenericDocumentFieldValidationStatus {
    var encodedString: String {
        switch (self){
            
        case .invalid:
            return  "Invalid"
        case .valid:
            return  "Valid"
        case .undefined:
            return "Undefined"
        @unknown default:
            return  "Undefined"
        }
    }
}

extension SBSDKGenericDocumentCommonFieldType {
    var encodedString: String {
        switch (self){
            
        case .ID:
            return "ID"
        case .surname:
            return "Surname"
        case .maidenName:
            return "MaidenName"
        case .givenNames:
            return "GivenNames"
        case .birthDate:
            return "BirthDate"
        case .nationality:
            return "Nationality"
        case .birthplace:
            return "Birthplace"
        case .expiryDate:
            return  "ExpiryDate"
        case .eyeColor:
            return "EyeColor"
        case .height:
            return "Height"
        case .issueDate:
            return "IssueDate"
        case .issuingAuthority:
            return "IssuingAuthority"
        case .address:
            return "Address"
        case .pseudonym:
            return "Pseudonym"
        case .MRZ:
            return "MRZ"
        case .countryCode:
            return "CountryCode"
        case .gender:
            return "Gender"
        case .signature:
            return "Signature"
        case .photo:
            return "Photo"
        case .validFromDate:
            return "ValidFromDate"
        case .routingNumber:
            return "RoutingNumber"
        case .accountNumber:
            return "AccountNumber"
        case .none:
            return "None"
        @unknown default:
            return "None"
        }
    }
}
