//
//  CallErrorHandler.swift
//  scanbot_sdk
//
//  Created by Yevgeniy Knizhnik on 15.06.21.
//

import Foundation

protocol CallErrorHandler {
    func returnArgumentsError(call: FlutterMethodCall, result: @escaping FlutterResult)
    func returnCustomError(call: FlutterMethodCall, message: String, result: @escaping FlutterResult)
    func returnUnknownCallError(call: FlutterMethodCall, result: @escaping FlutterResult)
}

extension CallErrorHandler {
    func returnArgumentsError(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.main.async {
            result(CallError.argumentsError(method: call.method, details: call.arguments))
        }
    }
    
    func returnUnknownCallError(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.main.async {
            result(CallError.unknownCallError(method: call.method))
        }
    }
    
    func returnCustomError(call: FlutterMethodCall, message: String, result: @escaping FlutterResult) {
        DispatchQueue.main.async {
            result(CallError.customError(message: message, details: call.arguments))
        }
    }
}
