import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:logging/logging.dart';
import 'camera_configuration.dart';
import '../scanbot_sdk_models.dart';

import '../mc_scanning_data.dart';
import 'live_detection.dart';

class MedicalCertificateCameraLiveDetector extends LiveDetector {
  /// The callback that will be called each time a medical certificate is detected.
  Function(MedicalCertificateResult result) mcListener;

  /// The callback that will be called when there are problems with a license.
  Function(SdkLicenseException exception) errorListener;

  @override
  DetectorType detectorType = DetectorType.mc;

  MedicalCertificateCameraLiveDetector(
      {required this.mcListener, required this.errorListener});

  Future<dynamic> _resultHandler(MethodCall methodCall) async {
    Map<String, dynamic> args = jsonDecode(methodCall.arguments);
    switch (methodCall.method) {
      case 'onMedicalCertificateDetected':
        if (!enabled) {
          return;
        }
        try {
          var scanningResult = MedicalCertificateResult.fromJson(args);
          mcListener(scanningResult);
        } catch (e) {
          Logger.root.severe(e);
        }
        return;
      case 'onLicenseError':
        errorListener.call(SdkLicenseException.fromJson(args));
        return;
      default:
    }
  }

  @override
  void registerResultHandler() {
    detectorChannel.setMethodCallHandler(_resultHandler);
  }
}
